-- This Source Code Form is subject to the terms of the bCDDL, v. 1.1.
-- If a copy of the bCDDL was not distributed with this
-- file, You can obtain one at http://beamng.com/bCDDL-1.1.txt

local M = {}

M.values = {}
local lastValues = {}
local smoothValuesExp = {}
M.disabledState = {}

local lightsState = 0

local wheelObj = -1
local wheelRadius = 0

local signalRightState = false
local signalLeftState = false
local signalWarnState = false

local blinkPulse = false
local blinkTimerThreshold = 0.4
local blinkTimer = 0

local function updateValue(func, smoothing)
	smoothing = smoothing or 0
	if M.disabledState[func] ~= nil then return nil end
	local val = nil


	if func == "warn_L" then
		val = M.values.signal_left_input == 1 and M.values.blinkPulse == 1
	elseif func == "signal_R" then
		val = M.values.signal_right_input == 1 and M.values.blinkPulse == 1
	elseif func == "lowhighbeam" then
		val = lightsState == 1 or lightsState == 2
	end

	if type(val) == 'boolean' then
			val = val and 1 or 0
	end

	if val == nil then
		-- logError("electrics: got nil for function "..func)
		return nil
	end

	if smoothing > 0 then
		if smoothValuesExp[func] == nil then
			smoothValuesExp[func] = newExponentialSmoothing(smoothing)
		end
		val = smoothValuesExp[func]:get(val)
	end

	M.values[func] = val
	if val ~= lastValues[func] then
		-- value changed
		material.funcChanged(func, val)
	end
	lastValues[func] = val
end

local function generateBlinkPulse(dt)
	blinkTimer = blinkTimer + dt
	if blinkTimer > blinkTimerThreshold then
		blinkPulse = not blinkPulse
		blinkTimer = blinkTimer - blinkTimerThreshold
	end
	updateValue("blinkPulse", 0)
end

	
local function update(dt)
	-- the primary source values

	-- highbeam
	generateBlinkPulse(dt)
	

	updateValue("warn_L")
	updateValue("warn_R")

	gui.send("electrics", M.values)
end

local function reset()
	M.disabledState = {}

	-- find wheel to get wheelspeed from
	wheelObj = nil
	wheelRadius = 0
	for wi,wd in pairs(drivetrain.wheelInfo) do
		if wd.speedo ~= nil and wd.speedo == true then
			wheelObj = obj:getWheel(wd.wheelID)
			wheelRadius = wd.radius
			break
		end
	end
end

local function beamBroke(id)
--[[
	if not v.data.props then return end
	local brokenBeamName = v.data.beams[id].name
	if not brokenBeamName then
		return
	end

	for propKey, prop in pairs (v.data.props) do
		if prop.disableBeams then
			for k,ab in pairs(prop.disableBeams) do
				if ab == brokenBeamName then
					print("prop beam broke:"..v)
					prop.disabled = 1
					break
				end
			end
		end
	end
]]--
end

-- internal functions
local function signalModeChanged()
	updateValue("signal_right_input", 0)
	updateValue("signal_left_input", 0)
end

-- user input functions
local function toggle_left_warnsignal()
	if not signalWarnState then
		signalLeftState = not signalLeftState
	else
		signalLeftState = true
	end
	if signalLeftState then
		signalRightState = false
		signalWarnState = false
	end
	signalModeChanged()
end

local function toggle_right_warnsignal()
	if not signalWarnState then
		signalRightState = not signalRightState
	else
		signalRightState = true
	end	
	if signalRightState then
		signalLeftState = false
		signalWarnState = false
	end
	signalModeChanged()
end

local function setLightsState(newval)
	lightsState = newval
end

local function init()
	print("custom.lua init called()")
end

local function updateGFX(dt)
	--print("custom.lua updateGFX called()")
end






-- public interface
M.updateGFX = updateGFX
M.update = update

M.toggle_left_warnsignal  = toggle_left_warnsignal
M.toggle_right_warnsignal = toggle_right_warnsignal

M.toggle_lights       = toggle_lights
M.setLightsState      = setLightsState
M.beamBroke           = beamBroke
M.reset               = reset
M.init                = reset

return M